@@ -37,11 +37,35 @@ def read(filename, variables=None, sel=None, full=False, jd=False):
 		*jd*: "Convert time variables to Julian dates (see [Aquarius Time](https://github.com/peterkuma/aquarius-time)) (`bool`)."
 	}}
 	"Supported formats": {{
+		CSV: `.csv`
 		DS: `.ds`
+		HDF5: "`.h5`, `.hdf5`, `.hdf`"
 		JSON: `.json`
-		NetCDF4: "`.nc`, `.nc4`, `.nc3`, `.netcdf`, `.hdf`, `.h5`"
+		NetCDF4: "`.nc`, `.nc4`, `.nc3`, `.netcdf`"
 	}}
 	returns: "Dataset (`dict`)."
+	examples: {{
+		"Read a file `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ print(d.keys())
+dict_keys(['.', 'temperature', 'time'])
+$ print(d['temperature'])
+[16. 18. 21.]
+$ d['.']['temperature']
+{'long_name': 'temperature', 'units': 'celsius', '.dims': ('time',), '.size': (3,), '.type': 'float64'}"
+		"Read a variable `temperature` at an index 0 of the dimension `time` from `dataset.nc`.":
+"$ d = ds.read('dataset.nc', 'temperature', sel={'time': 0})
+$ d.keys()
+dict_keys(['.', 'temperature'])
+$ print(d['temperature'])
+16.0"
+		"Read only the metadata of `dataset.nc`.":
+"$ d = ds.read('dataset.nc', [], full=True)
+$ d.keys()
+dict_keys(['.'])
+$ print(d['.'])
+{'.': {'title': 'Temperature data'}, 'temperature': {'long_name': 'temperature', 'units': 'celsius', '.dims': ('time',), '.size': (3,), '.type': 'float64'}, 'time': {'long_name': 'time', 'units': 's', '.dims': ('time',), '.size': (3,), '.type': 'int64'}}"
+	}}
 	'''
 	check(filename, 'filename', [str, bytes, os.PathLike])
 	check(variables, 'variables', [str, [list, str], [tuple, str], None])
@@ -73,7 +97,35 @@ def readdir(dirname, variables=None, merge=None, warnings=[], **kwargs):
 		*warnings*: "A list to be populated with warnings (`list`)."
 		...: "Optional keyword arguments passed to **[read](#read)**."
 	}}
+	"Supported formats": {{
+		CSV: `.csv`
+		DS: `.ds`
+		HDF5: "`.h5`, `.hdf5`, `.hdf`"
+		JSON: `.json`
+		NetCDF4: "`.nc`, `.nc4`, `.nc3`, `.netcdf`"
+	}}
 	returns: "A list of datasets (`list` of `dict`) if *merge* is `None` or a merged dataset (`dict`) if *merge* is a dimension name."
+	examples: {{
+		"Read datasets `dataset1.nc` and `dataset2.nc` in the current directory (`.`).":
+"$ ds.write('dataset1.nc', { 'time': [1, 2, 3], 'temperature': [16., 18., 21.], '.': {
+	'time': { '.dims': ['time'] },
+	'temperature': { '.dims': ['time'] },
+}})
+$ ds.write('dataset2.nc', { 'time': [4, 5, 6], 'temperature': [23., 25., 28.], '.': {
+	'time': { '.dims': ['time'] },
+	'temperature': { '.dims': ['time'] },
+}})
+$ dd = ds.readdir('.')
+$ for d in dd: print(d['time'])
+[1 2 3]
+[4 5 6]"
+		"Read datasets in the current directory and merge them by a dimension `time`.":
+"$ d = ds.readdir('.', merge='time')
+$ print(d['time'])
+[1 2 3 4 5 6]
+$ print(d['temperature'])
+[16. 18. 21. 23. 25. 28.]"
+	}}
 	'''
 	check(dirname, 'dirname', [str, bytes, os.PathLike])
 	check(variables, 'variables', [str, [list, str], [tuple, str], None])
@@ -118,11 +170,30 @@ def write(filename, d):
 		*d*: "Dataset (`dict`)."
 	}}
 	"Supported formats": {{
-		NetCDF4: "`.nc`, `.nc4`, `.netcdf`"
-		JSON: `.json`
+		CSV: `.csv`
 		DS: `.ds`
+		HDF5: "`.h5`, `.hdf5`, `.hdf`"
+		JSON: `.json`
+		NetCDF4: "`.nc`, `.nc4`, `.netcdf`"
 	}}
 	returns: `None`
+	examples: {{
+		"Write a dataset to a file `dataset.nc`.":
+"$ ds.write('dataset.nc', {
+	'time': [1, 2, 3],
+	'temperature': [16. 18. 21.],
+})"
+		"Write a dataset with metadata to a file `dataset.nc`.":
+"$ ds.write('dataset.nc', {
+	'time': [1, 2, 3],
+	'temperature': [16. 18. 21.],
+	'.': {
+		'.': { 'title': 'Temperature data' },
+		'time': { '.dims': ['time'] },
+		'temperature': { '.dims': ['time'], 'units': 'degree_celsius' },
+	}
+})"
+	}}
 	'''
 	check(filename, 'filename', [str, bytes, os.PathLike])
 	check(d, 'd', dict)
