@@ -1,12 +1,13 @@
+import types
+import fnmatch
+import numpy as np
+import copy as copy_
+import datetime as dt
+from warnings import warn
+from collections import Mapping, Iterable
 import ds_format as ds
 from ds_format.misc import check
-from collections import Mapping, Iterable
-import copy as copy_
-import numpy as np
-import datetime as dt
-import fnmatch
 from . import misc
-from warnings import warn
 
 #
 # Private variables.
@@ -140,6 +141,19 @@ def attr(d, attr, *value, var=None):
 		*var*: "Variable name (`str`) to get or set a variable attribute, or `None` to get or set a dataset attribute."
 	}}
 	returns: "Attribute value if *value* is not set, otherwise `None`."
+	examples: {{
+		"Get an attribute `long_name` of a variable `temperature` in `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+ds.attr(d, 'long_name', var='temperature')
+'temperature'"
+		"Get a dataset attribute `title` of `dataset.nc`.":
+"$ ds.attr(d, 'title')
+'Temperature data'"
+		"Set an attribute `units` of a variable `temperature` to `K`.":
+"$ ds.attr(d, 'units', 'K', var='temperature')
+$ ds.attr(d, 'units', var='temperature')
+'K'"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(attr, 'attr', str)
@@ -157,7 +171,7 @@ def attr(d, attr, *value, var=None):
 def attrs(d, var=None, *value):
 	'''
 	title: attrs
-	caption: "Get variable or dataset attributes."
+	caption: "Get or set variable or dataset attributes."
 	usage: "`attrs`(*d*, *var*=`None`, \**value*)"
 	arguments: {{
 		*d*: "Dataset (`dict`)."
@@ -167,6 +181,19 @@ def attrs(d, var=None, *value):
 		*var*: "Variable name (`str`) or `None` to get dataset attributes."
 	}}
 	returns: "Attributes (`dict`)."
+	examples: {{
+		"Get attributes of a variable `temperature` in a dataset `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.attrs(d, 'temperature')
+{'long_name': 'temperature', 'units': 'celsius'}"
+		"Get dataset attributes.":
+"$ ds.attrs(d)
+{'title': 'Temperature data'}"
+		"Set attributes of a variable `temperature`.":
+"$ ds.attrs(d, 'temperature', {'long_name': 'new temperature', 'units': 'K'})
+$ ds.attrs(d, 'temperature')
+{'long_name': 'new temperature', 'units': 'K'}"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(var, 'var', [str, None])
@@ -195,6 +222,16 @@ def dim(d, dim, full=False):
 		*full*: "Return dimension size also for a dimension for which no variable data are defined, i.e. it is only defined in dataset metadata."
 	}}
 	returns: "Dimension size or 0 if the dimension does not exist (`int`)."
+	examples: {{
+		"Get the size of a dimension `time` in `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.dim(d, 'time')
+3"
+		"Get the size of a dimension `time` in `dataset.nc` without reading data.":
+"$ d = ds.read('dataset.nc', full=True)
+$ ds.dim(d, 'time', full=True)
+3"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(dim, 'dim', str)
@@ -222,6 +259,18 @@ def dims(d, var=None, *value, full=False, size=False):
 		*size*: "Return a dictionary containing dimension sizes instead of a list."
 	}}
 	returns: "If *size* is False, a list of dataset or variable dimension names (`list` of `str`). If *size* is True, a dictionary of dataset or variable dimension names and sizes (`dict`), where a key is a dimension name (`str`) and the value is the dimension size (`int`). The order of keys in the dictionary is not guaranteed. Dataset dimensions are the dimensions of all variables together."
+	examples: {{
+		"Get dimensions of a dataset `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.dims(d)
+['time']"
+		"Get dimension sizes.":
+"$ ds.dims(d, size=True)
+{'time': 3}"
+		"Get dimensions of a variable `temperature`.":
+"$ ds.dims(d, 'temperature')
+['time']"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(var, 'var', [str, None])
@@ -301,6 +350,12 @@ def find(d, what, name, var=None):
 		*var*: "Variable name (`str`) or `None`. Applies only if *what* is \\"attr\\". If not `none`, *name* is a variable attribute name, otherwise it is a dataset attribute name."
 	}}
 	returns: "A variable, dimension or attribute name matching the pattern, or *name* if no matching name is found (`str`)."
+	examples: {{
+		"Find a variable matching the glob pattern `temp*` in a dataset `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.find(d, 'var', 'temp*')
+'temperature'"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(what, 'what', str)
@@ -329,6 +384,12 @@ def findall(d, what, name, var=None):
 		*var*: "Variable name (`str`) or `None`. Applies only if *what* is \\"attr\\". If not `none`, *name* is a variable attribute name, otherwise it is a dataset attribute name."
 	}}
 	returns: "A list of variables, dimensions or attributes matching the pattern, or [*name*] if no matching names are found (`list` of `str`)."
+	examples: {{
+		"Find all variables matching the glob pattern `t*` in a dataset `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.findall(d, 'var', 't*')
+['temperature', 'time']"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(what, 'what', str)
@@ -358,11 +419,27 @@ def group_by(d, dim, group, func):
 		*func*: "Group function (`function`). *func*(*y*, axis=*i*) is called for each subset *y*, where *i* is the index of the dimension."
 	}}
 	returns: `None`
+	examples: {{
+		"Calculate mean along a dimension `time` for a group where time <= 2 and a group where time > 2.":
+"$ d = {
+	'time': np.array([1., 2., 3., 4.]),
+	'temperature': np.array([1., 3., 4., 6.]),
+	'.': {
+		'time': { '.dims': ['time'] },
+		'temperature': { '.dims': ['time'] },
+	}
+}
+$ ds.group_by(d, 'time', d['time'] > 2,  np.mean)
+$ print(d['time'])
+[1.5 3.5]
+$ print(d['temperature'])
+[1.5 3.5]"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(dim, 'dim', str)
 	check(group, 'group', [np.ndarray, list, tuple])
-	check(func, 'func', function)
+	check(func, 'func', types.FunctionType)
 	groups = sorted(list(set(group)))
 	vars_ = ds.vars(d)
 	n = len(groups)
@@ -401,6 +478,22 @@ def merge(dd, dim, new=None, variables=None):
 		*variables*: "Variables to merge along a new dimension (`list`) or `None` for all variables."
 	}}
 	returns: "A dataset (`dict`)."
+	examples: {{
+		"Merge datasets `d1` and `d2` along a dimension `time`.":
+"$ d1 = {'time': [1, 2, 3], 'temperature': [16., 18., 21.], '.': {
+	'time': { '.dims': ['time'] },
+	'temperature': { '.dims': ['time'] },
+}}
+$ d2 = { 'time': [4, 5, 6], 'temperature': [23., 25., 28.], '.': {
+	'time': { '.dims': ['time'] },
+	'temperature': { '.dims': ['time'] },
+}}
+$ d = ds.merge([d1, d2], 'time')
+$ print(d['time'])
+[1 2 3 4 5 6]
+$ print(d['temperature'])
+[16. 18. 21. 23. 25. 28.]"
+	}}
 	'''
 	check(dd, 'dd', [[list, dict], [tuple, dict]])
 	check(dim, 'dim', str)
@@ -426,60 +519,73 @@ def merge(dd, dim, new=None, variables=None):
 			dx['.']['.'].update(d['.']['.'])
 	return dx
 
-def meta(d, var=None, meta=None, create=False):
+def meta(d, var=None, *value, create=False):
 	'''
 	title: meta
 	aliases: { get_meta }
 	caption: "Get or set dataset or variable metadata."
-	usage: "`meta`(*d*, *var*=`None`, *meta*=`None`, *create*=`False`)"
+	usage: "`meta`(*d*, *var*=`None`, \**value*, *create*=`False`)"
 	arguments: {{
 		*d*: "Dataset (`dict`)."
 	}}
 	options: {{
 		*var*: "Variable name (`str`), or `None` to get dataset metadata, or an empty string to get dataset attributes."
-		*meta*: "Metadata to set (`dict`) or `None` to get metadata."
-		*create*: "Create (modifyable/bound) metadata dictionary in the dataset if not defined (`bool`). If `False`, the returned dictionary is an empty unbound dictionary if not present in the dataset."
+		*value*: "Metadata to set (`dict`) or `None` to get metadata."
+		*create*: "Create (modifyable/bound) metadata dictionary in the dataset if not defined (`bool`). If `False`, the returned dictionary is an empty unbound dictionary if it is not already present in the dataset."
 	}}
 	returns: "Metadata (`dict`)."
+	examples: {{
+		"Get metadata of a dataset `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ print(ds.meta(d))
+{'.': {'title': 'Temperature data'}, 'temperature': {'long_name': 'temperature', 'units': 'celsius', '.dims': ('time',), '.size': (3,), '.type': 'float64'}, 'time': {'long_name': 'time', 'units': 's', '.dims': ('time',), '.size': (3,), '.type': 'int64'}}"
+		"Get metadata of a variable `temperature`.":
+"$ ds.meta(d, 'temperature')
+{'long_name': 'temperature', 'units': 'celsius', '.dims': ('time',), '.size': (3,), '.type': 'float64'}"
+		"Set metadata of a variable `temperature`.":
+"$ ds.meta(d, 'temperature', { '.dims': ['new_time'], 'long_name': 'new temperature', 'units': 'K'})
+$ ds.meta(d, 'temperature')
+ds.meta(d, 'temperature', { '.dims': ['new_time'], 'long_name': 'new temperature', 'units': 'K'})"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(var, 'var', [str, None])
-	check(meta, 'meta', [[dict, str], None])
 	var_e = ds.escape(var)
 
-	if meta is not None:
+	if len(value) == 0:
 		if var is None:
-			d['.'] = meta
+			if '.' in d:
+				return d['.']
+			if create:
+				d['.'] = {}
+				return d['.']
+			return {}
+		elif var == '':
+			meta = ds.meta(d, create=create)
+			if '.' in meta:
+				return meta['.']
+			if create:
+				meta['.'] = {}
+				return meta['.']
+			return {}
+		else:
+			meta = ds.meta(d, create=create)
+			if var_e in meta:
+				return meta[var_e]
+			if create:
+				meta[var_e] = {}
+				return meta[var_e]
+			require(d, 'var', var, full=True)
+			return {}
+	elif len(value) == 1:
+		check(value[0], 'value', [dict, str])
+		if var is None:
+			d['.'] = value[0]
 		else:
 			ds_meta = ds.meta(d, create=True)
-			ds_meta[var_e] = meta
-		return meta
-
-	if var is None:
-		if '.' in d:
-			return d['.']
-		if create:
-			d['.'] = {}
-			return d['.']
-		return {}
-
-	if var == '':
-		meta = ds.meta(d, create=create)
-		if '.' in meta:
-			return meta['.']
-		if create:
-			meta['.'] = {}
-			return meta['.']
-		return {}
-
-	meta = ds.meta(d, create=create)
-	if var_e in meta:
-		return meta[var_e]
-	if create:
-		meta[var_e] = {}
-		return meta[var_e]
-	require(d, 'var', var, full=True)
-	return {}
+			ds_meta[var_e] = value[0]
+	else:
+		raise TypeError('only one value argument is expected')
 
 meta.aliases = ['get_meta']
 get_meta = meta
@@ -496,6 +602,15 @@ def rename(d, old, new):
 		*new*: "New variable name (`str`) or `None` to remove the variable."
 	}}
 	returns: `None`
+	examples: {{
+		"Rename a variable `temperature` to `new_temperature` in a dataset read from `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.vars(d)
+['temperature', 'time']
+$ ds.rename(d, 'temperature', 'new_temperature')
+$ ds.vars(d)
+['new_temperature', 'time']"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(old, 'old', str)
@@ -526,6 +641,21 @@ def rename_attr(d, old, new, var=None):
 		*var*: "Variable name (`str`) to rename a variable attribute or `None` to rename a dataset attribute."
 	}}
 	returns: `None`
+	examples: {{
+		"Rename an attribute `units` of a variable `temperature` to `new_units` in a dataset read from `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.attrs(d, 'temperature')
+{'long_name': 'temperature', 'units': 'celsius'}
+$ ds.rename_attr(d, 'units', 'new_units', var='temperature')
+$ ds.attrs(d, 'temperature')
+{'long_name': 'temperature', 'new_units': 'celsius'}"
+		"Rename a dataset attribute `title` to `new_title`.":
+"$ ds.attrs(d)
+{'title': 'Temperature data'}
+$ ds.rename_attr(d, 'title', 'new_title')
+$ ds.attrs(d)
+{'new_title': 'Temperature data'}"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(old, 'old', str)
@@ -541,7 +671,7 @@ def rename_attr(d, old, new, var=None):
 
 def rename_dim(d, old, new):
 	'''
-	title: rename
+	title: rename_dim
 	caption: "Rename a dimension."
 	usage: "`rename_dim`(*d*, *old*, *new*)"
 	arguments: {{
@@ -550,6 +680,15 @@ def rename_dim(d, old, new):
 		*new*: "New dimension name (`str`)."
 	}}
 	returns: `None`
+	examples: {{
+		"Rename a dimension `time` to `new_time` in a dataset read from `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.dims(d)
+['time']
+$ ds.rename_dim(d, 'time', 'new_time')
+$ ds.dims(d)
+['new_time']"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(old, 'old', str)
@@ -582,6 +721,12 @@ def require(d, what, name, var=None, full=False):
 		*full*: "Also look for items which are defined only in dataset metadata (`bool`)."
 	}}
 	returns: "`true` if the required item is defined in the dataset, otherwise `false` or raises an exception depending on the mode."
+	examples: {{
+		"Require that a variable `temperature` is defined in a dataset read from `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.require(d, 'var', 'temperature')
+True"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(what, 'what', str)
@@ -622,6 +767,15 @@ def rm(d, var):
 		*var*: "Variable name (`str`)."
 	}}
 	returns: `None`
+	examples: {{
+		"Remove a variable `temperature` in a dataset read from `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.vars(d)
+['temperature', 'time']
+$ ds.rm(d, 'temperature')
+$ ds.vars(d)
+['time']"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(var, 'var', str)
@@ -641,6 +795,21 @@ def rm_attr(d, attr, var=None):
 		*var*: "Variable name (`str`) to remove a variable attribute or `None` to remove a dataset attribute."
 	}}
 	returns: `None`
+	examples: {{
+		"Remove an attribute `long_name` of a variable `temperature` in a dataset read from `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.attrs(d, 'temperature')
+{'long_name': 'temperature', 'units': 'celsius'}
+$ ds.rm_attr(d, 'long_name', var='temperature')
+$ ds.attrs(d)
+{'title': 'Temperature data'}"
+		"Remove a dataset attribute `title` in a dataset read from `dataset.nc`.":
+"$ ds.attrs(d)
+{'title': 'Temperature data'}
+$ ds.rm(d, 'title')
+$ ds.attrs(d)
+{}"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(attr, 'attr', str)
@@ -654,11 +823,26 @@ def select(d, sel):
 	title: select
 	caption: "Filter dataset by a selector."
 	usage: "`select`(*d*, *sel*)"
+	desc: "The function subsets data of all variables in a dataset *d* by a selector *sel*. Data can be subset by a mask or a list of indexes along one or more dimensions."
 	arguments: {{
 		*d*: "Dataset (`dict`)."
-		*sel*: "Selector (`dict`). Selector is a dictionary where each key is a dimension name and value is a mask to apply along the dimension or a list of indexes."
+		*sel*: "Selector (`dict`). Selector is a dictionary where the key is a dimension name (`str`) and the value is a mask, a list of indexes (`list` or `np.array`) or an index (`int`) to subset by along the dimension."
 	}}
 	returns: `None`
+	examples: {{
+		"Subset index 0 a along dimension `time` in a dataset read from `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.var(d, 'temperature')
+print(ds.var(d, 'temperature'))
+$ ds.select(d, {'time': 0})
+$ ds.var(d, 'temperature')
+16"
+		"Subset by a mask along a dimension `time` in a dataset read from `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.select(d, {'time': [False, True, True]})
+$ ds.var(d, 'temperature')
+[18. 21.]"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(sel, 'sel', [[dict, str]])
@@ -676,16 +860,22 @@ def size(d, var):
 		*var*: "Variable name (`str`)."
 	}}
 	returns: "Variable size (`list`) or `None` if not defined."
+	examples: {{
+		"Get the size of a variable `temperature` in `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.size(d, 'temperature')
+[3]"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(var, 'var', str)
 	if require(d, 'var', var, full=True):
 		if var in ds.vars(d):
 			data = ds.var(d, var)
-			return None if data is None else data.shape
+			return None if data is None else list(data.shape)
 		else:
 			meta = ds.meta(d, var)
-			return meta.get('.size')
+			return list(meta.get('.size'))
 	else:
 		return None
 
@@ -701,6 +891,18 @@ def type_(d, var, *value):
 		*value*: "Variable type (`str`). One of: `float32` and `float64` (32-bit and 64-bit floating-point number, resp.), `int8` `int16`, `int32` and `int64` (8-bit, 16-bit, 32-bit and 64-bit integer, resp.), `uint8`, `uint16`, `uint32` and `uint64` (8-bit, 16-bit, 32-bit and 64-bit unsigned integer, resp.), `bool` (boolean), `str` (string) and `unicode` (Unicode)."
 	}}
 	returns: "Variable type (`str`) or `None` if not defined."
+	examples: {{
+		"Get the type of a variable `temperature` in `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.type(d, 'temperature')
+'float64'"
+		"Set the type of a variable `temperature` to `int64`.":
+"$ ds.type(d, 'temperature', 'int64')
+$ ds.type(d, 'temperature')
+'int64'
+$ print(ds.var(d, 'temperature'))
+[16 18 21]"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(var, 'var', str)
@@ -738,7 +940,17 @@ def var(d, var, *value):
 		*var*: "Variable name (`str`)."
 		*value*: "Variable data. If supplied, set variable data, otherwise get variable data."
 	}}
-	returns: "Variable data (`np.ndarray` or `np.generic`) or `None` if the variable data are not defined or `value` is supplied. If the variable data are a `list` or `tuple`, they are converted to `np.ndarray`, or to `np.ma.MaskedArray` if they contain `None`, which is masked. If the variable data are `int`, `float, `bool`, `str` or `bytes`, they are converted to `np.generic`. Raises ValueError if the output dtype is not one of `float32`, `float64`, `int8`, `int16`, `int32`, `int64`, `uint8`, `uint16`, `uint32`, `uint64`, `bool`, `bytes<n>`, `str<n>`, or `object` for which all items are an instance of `str` or `bytes`."
+	returns: "Variable data (`np.ndarray` or `np.generic`) or `None` if the variable data are not defined or `value` is supplied. If the variable data are a `list` or `tuple`, they are converted to `np.ndarray`, or to `np.ma.MaskedArray` if they contain `None`, which is masked. If the variable data are `int`, `float`, `bool`, `str` or `bytes`, they are converted to `np.generic`. Raises `ValueError` if the output dtype is not one of `float32`, `float64`, `int8`, `int16`, `int32`, `int64`, `uint8`, `uint16`, `uint32`, `uint64`, `bool`, `bytes<n>`, `str<n>`, or `object` for which all items are an instance of `str` or `bytes`."
+	examples: {{
+		"Get data of a variable `temperature` in a dataset `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ print(ds.var(d, 'temperature'))
+[16. 18. 21.]"
+		"Set data of a variable `temperature`.":
+"$ ds.var(d, 'temperature', [17, 18, 22])
+$ ds.var(d, 'temperature')
+array([17, 18, 22])"
+	}}
 	'''
 	check(d, 'd', dict)
 	check(var, 'var', str)
@@ -783,6 +995,16 @@ def vars_(d, full=False):
 		*full*: "Also return variable names which are only defined in the metadata."
 	}}
 	returns: "Variable names (`list` of `str`)."
+	examples: {{
+		"List variables in a dataset `dataset.nc`.":
+"$ d = ds.read('dataset.nc')
+$ ds.vars(d)
+['temperature', 'time']"
+		"List variables in a dataset `dataset.nc` without reading the data.":
+"$ d = ds.read('dataset.nc', [], full=True)
+$ ds.vars(d, full=True)
+['temperature', 'time']"
+	}}
 	'''
 	check(d, 'd', dict)
 	meta = ds.meta(d)
